"use strict";
const Common = require("@surface/common");
const FS = require("fs");
const Path = require("path");
class CodeSplitterPlugin {
    constructor(options) {
        this._options = options;
    }
    apply(compiler) {
        const self = this;
        compiler.plugin('make', function (compilation, callback) {
            if (!self._options.entries)
                throw new Error('Entries not specified');
            if (!this.options.context)
                throw new Error('Context can\'t be null');
            let file = Path.join(Common.lookUp(this.options.context, 'node_modules'), '@surface', 'lazy-loader', 'index.js');
            for (let entry of self._options.entries) {
                let paths = self.getPaths(Path.resolve(this.options.context, entry));
                let content = self.writeHeader() + '\n';
                for (let path of paths) {
                    if (path.name == 'index')
                        content += self.writeEntry(`${entry}/${path.dir.split(Path.sep).pop()}`, Path.format(path)) + '\n';
                    else
                        content += self.writeEntry(`${entry}/${path.name}`, Path.format(path)) + '\n';
                }
                content += self.writeFooter();
                FS.writeFileSync(file, content);
            }
            callback();
        });
    }
    getPaths(entry) {
        let result = [];
        if (!FS.existsSync(entry))
            throw new Error('Path not exists');
        if (!FS.lstatSync(entry).isDirectory())
            throw new Error('Path is not a directory');
        for (let source of FS.readdirSync(entry)) {
            let currentPath = Path.join(entry, source);
            if (FS.lstatSync(currentPath).isDirectory()) {
                ['index.ts', 'index.js'].forEach(fileName => {
                    let file = Path.join(currentPath, fileName);
                    if (FS.existsSync(file))
                        result.push(Path.parse(file));
                });
            }
            else
                result.push(Path.parse(currentPath));
        }
        return result;
    }
    writeEntry(name, path) {
        name = name.replace('./', '');
        let result = [
            `        case '${name}':`,
            `            return import(/* webpackChunkName: '${name}' */ '${path.replace(/\\/g, '\\\\')}');`
        ].join('\n');
        return result;
    }
    writeFooter() {
        let result = [
            '        default:',
            '            return Promise.reject("path not found");',
            '    }',
            '}',
        ].join('\n');
        return result;
    }
    writeHeader() {
        let result = [
            'export default function(name)',
            '{',
            '    switch (name)',
            '    {',
        ].join('\n');
        return result;
    }
}
module.exports = CodeSplitterPlugin;
//# sourceMappingURL=index.js.map